/* SAM3XA PIO Header
   (c) S. Raase, 2015 */
#ifndef _DEVICES_PIO_H_
#define _DEVICES_PIO_H_

typedef struct {
	reg32_wo  PIO_PER;	/* PIO Enable Register */
	reg32_wo  PIO_PDR;	/* PIO Disable Register */
	reg32_ro  PIO_PSR;	/* PIO Status Register */
	reg32_und Reserved1;	/* Reserved */
	reg32_wo  PIO_OER;	/* Output Enable Register */
	reg32_wo  PIO_ODR;	/* Output Disable Register */
	reg32_ro  PIO_OSR;	/* Output Status Register */
	reg32_und Reserved2;	/* Reserved */
	reg32_wo  PIO_IFER;	/* Glitch Input Filter Enable Register */
	reg32_wo  PIO_IFDR;	/* Glitch Input Filter Disable Register */
	reg32_ro  PIO_IFSR;	/* Glitch Input Filter Status Register */
	reg32_und Reserved3;	/* Reserved */
	reg32_wo  PIO_SODR;	/* Set Output Data Register */
	reg32_wo  PIO_CODR;	/* Clear Output Data Register */
	reg32_rw  PIO_ODSR;	/* Output Data Status Register */
	reg32_ro  PIO_PDSR;	/* Pin Data Status Register */
	reg32_wo  PIO_IER;	/* Interrupt Enable Register */
	reg32_wo  PIO_IDR;	/* Interrupt Disable Register */
	reg32_ro  PIO_IMR;	/* Interrupt Mask Register */
	reg32_ro  PIO_ISR;	/* Interrupt Status Register */
	reg32_wo  PIO_MDER;	/* Multi-driver Enable Register */
	reg32_wo  PIO_MDDR;	/* Multi-driver Disable Register */
	reg32_ro  PIO_MDSR;	/* Multi-driver Status Register */
	reg32_und Reserved4;	/* Reserved */
	reg32_wo  PIO_PUDR;	/* Pull-up Disable Register */
	reg32_wo  PIO_PUER;	/* Pull-up Enable Register */
	reg32_ro  PIO_PUSR;	/* Pad Pull-up Status Register */
	reg32_und Reserved5;	/* Reserved */
	reg32_rw  PIO_ABSR;	/* Peripheral AB Select Register */
	reg32_und Reserved6[3]; /* Reserved */
	reg32_wo  PIO_SCIFSR;	/* System Clock Glitch Input Filter Select Register */
	reg32_wo  PIO_DIFSR;	/* Debouncing Input Filter Select Register */
	reg32_ro  PIO_IFDGSR;	/* Glitch or Debouncing Input Filter Clock Selection Status Register */
	reg32_rw  PIO_SCDR;	/* Slow Clock Divider Debouncing Register */
	reg32_und Reserved7[4];	/* Reserved */
	reg32_wo  PIO_OWER;	/* Output Write Enable */
	reg32_wo  PIO_OWDR;	/* Output Write Disable */
	reg32_ro  PIO_OWSR;	/* Output Write Status Register */
	reg32_und Reserved8;	/* Reserved */
	reg32_wo  PIO_AIMER;	/* Additional Interrupt Modes Enable Register */
	reg32_wo  PIO_AIMDR;	/* Additional Interrupt Modes Disables Register */
	reg32_ro  PIO_AIMMR;	/* Additional Interrupt Modes Mask Register */
	reg32_und Reserver9;	/* Reserved */
	reg32_wo  PIO_ESR;	/* Edge Select Register */
	reg32_wo  PIO_LSR;	/* Level Select Register */
	reg32_ro  PIO_ELSR;	/* Edge/Level Status Register */
	reg32_und Reserved10;	/* Reserved */
	reg32_wo  PIO_FELLSR;	/* Falling Edge/Low Level Select Register */
	reg32_wo  PIO_REHLSR;	/* Rising Edge/ High Level Select Register */
	reg32_ro  PIO_FRLHSR;	/* Fall/Rise - Low/High Status Register */
	reg32_und Reserved11;	/* Reserved */
	reg32_ro  PIO_LOCKSR;	/* Lock Status */
	reg32_rw  PIO_WPMR;	/* Write Protect Mode Register */
	reg32_ro  PIO_WPSR;	/* Write Protect Status Register */
} regs_pio;

#define PIO_P0  (1 << 0)
#define PIO_P1  (1 << 1)
#define PIO_P2  (1 << 2)
#define PIO_P3  (1 << 3)
#define PIO_P4  (1 << 4)
#define PIO_P5  (1 << 5)
#define PIO_P6  (1 << 6)
#define PIO_P7  (1 << 7)
#define PIO_P8  (1 << 8)
#define PIO_P9  (1 << 9)
#define PIO_P10 (1 << 10)
#define PIO_P11 (1 << 11)
#define PIO_P12 (1 << 12)
#define PIO_P13 (1 << 13)
#define PIO_P14 (1 << 14)
#define PIO_P15 (1 << 15)
#define PIO_P16 (1 << 16)
#define PIO_P17 (1 << 17)
#define PIO_P18 (1 << 18)
#define PIO_P19 (1 << 19)
#define PIO_P20 (1 << 20)
#define PIO_P21 (1 << 21)
#define PIO_P22 (1 << 22)
#define PIO_P23 (1 << 23)
#define PIO_P24 (1 << 24)
#define PIO_P25 (1 << 25)
#define PIO_P26 (1 << 26)
#define PIO_P27 (1 << 27)
#define PIO_P28 (1 << 28)
#define PIO_P29 (1 << 29)
#define PIO_P30 (1 << 30)
#define PIO_P31 (1 << 31)

#endif /* _DEVICES_PIO_H_ */
