/* SAM3X/A Flash Linker Script
   S. Raase, 2015 */
OUTPUT_FORMAT("elf32-littlearm")
OUTPUT_ARCH(arm)

/* memory areas */
MEMORY {
	rom (rx)  : ORIGIN = 0x00080000, LENGTH = 0x00080000 /* 512K Flash */
	ram (rwx) : ORIGIN = 0x20070000, LENGTH = 0x00018000 /* 96K  SRAM*/
}

/* put stack at end of memory */
_stack_end = ORIGIN(ram) + LENGTH(ram);

/* section definitions */
SECTIONS {
	/* vector table */
	.vectors : {
		. = ALIGN(4);
		KEEP(*(.vectors .vectors.*))
	} > rom

	/* program code and read-only data */
	.text : {
		. = ALIGN(4);
		_text_start = .;
		*(.text .text.*)
		*(.rodata .rodata.*)
		_text_end = .;
	} > rom

	. = ALIGN(4);
	_ro_end = .;

	/* initialized data */
	.data : AT(_text_end) {
		. = ALIGN(4);
		_data_start = .;
		*(.data .data.*)
		. = ALIGN(4);
		_data_end = .;
	} > ram

	/* uninitialized data */
	.bss (NOLOAD) : {
		. = ALIGN(4);
		_bss_start = .;
		*(.bss .bss.*)
		. = ALIGN(4);
		_bss_end = .;
	} > ram

	. = ALIGN(4);
	_final = .;
}
