/* Main Program File */
#include <sam3xa.h>

static void SystemInit(void)
{
	/* disable watchdog */
	WDT->WDT_MR = WDT_MR_WDDIS;

	/* set SCK (slow clock) to external 32.768 kHz crystal */
	SUPC->SUPC_CR = SUPC_CR_KEY(0xA5) | SUPC_CR_XTALSEL;
	while(!(SUPC->SUPC_SR & SUPC_SR_OSCSEL));

	/* enable Main Crystal Oscillator as MAINCK */
	PMC->CKGR_MOR = CKGR_MOR_MOSCSEL | CKGR_MOR_KEY(0x37) |
		CKGR_MOR_MOSCXTST(255) | CKGR_MOR_MOSCXTEN;
	while(!(PMC->PMC_SR & PMC_SR_MOSCXTS));

	/* set MCK as MAINCK, with prescaler zero */
	PMC->PMC_MCKR = PMC_MCKR_PRES(0) | PMC_MCKR_CSS(1);
	while(!(PMC->PMC_SR & PMC_SR_MCKRDY));

	/* now, the CPU should run at 12 MHz;
	   MAINF in CKGR_MCFR should read ca. 5859;
	   (lower 2 bytes at 0x400E0624 should read ca. 0x16E3) */
}

void main(void)
{
	SystemInit();

	/* enable clock for PIO B */
	PMC->PMC_PCER0 = (1 << 12);	/* see "Product Mapping" */

	/* configure PB27 (LED 'L') as output */
	PIOB->PIO_PER  = PIO_P27;
	PIOB->PIO_OER  = PIO_P27;
	PIOB->PIO_PUDR = PIO_P27;

	/* blink LED */
	while(1) {
		PIOB->PIO_SODR = PIO_P27;
		for(volatile int i = 0; i < 100000; i++);
		PIOB->PIO_CODR = PIO_P27;
		for(volatile int i = 0; i < 100000; i++);
	}
}

